"""Create Manim Editor project."""
import os
import json
from typing import Any, Dict, Tuple, List

from .manim_loader import get_scenes


def create_project_dir(project_name: str) -> Tuple[bool, str]:
    """Ensure existence of project dir.
    Return True if success and message for the user."""
    print(f"Creating project '{project_name}'.")
    # TODO: valid dir name check for Windows
    if project_name == "":
        return False, "The project name can't be empty."
    if os.path.exists(project_name):
        # empty dirs are ok
        if len(os.listdir(project_name)) == 0:
            return True, f"The project directory '{project_name}' has already been created. It will be used."
        return False, f"The project name '{project_name}' points to a filled directory. If this is a project, you can open it instead."
    try:
        os.mkdir(project_name)
    except FileNotFoundError:
        return False, f"Creation of project directory '{project_name}' failed."
    return True, f"Created new project directory '{project_name}'."


class SectionId:
    def __init__(self, scene_id: int, section_id: int):
        self.scene_id = scene_id
        self.section_id = section_id


def populate_project(project_name: str, section_ids: List[SectionId]) -> bool:
    """Create project JSON file, copy selected section video files and create thumbnails.
    Return False at failure."""
    print(f"Populating project '{project_name}'.")
    # TODO: could theoretically be cached
    scenes = get_scenes()
    # select sections according to ids set by frontend
    sections = [scenes[section.scene_id].sections[section.section_id] for section in section_ids]

    project: List[Dict[str, Any]] = []

    # prepare section
    for id, section in enumerate(sections):
        if not section.set_project(project_name, id):
            return False
        project.append(section.get_dict())

    # write project file
    with open(os.path.join(project_name, "project.json"), "w") as file:
        json.dump(project, file, indent=4)
    return True



from .app import create_app
import click
import logging
from waitress import serve

from .config import Config
from .editor import set_config


def run_normal() -> None:
    set_config(Config)
    app = create_app(Config)
    # disable logging for every get/post request
    log = logging.getLogger("werkzeug")
    log.setLevel(logging.ERROR)
    print("Starting Manim Editor. Open http://localhost:5000 in a browser. (Press CTRL+C to quit)")
    serve(app, host="0.0.0.0", port=5000)


def run_debug() -> None:
    Config.set("FFMPEG_LOGLEVEL", "info")
    set_config(Config)
    app = create_app(Config)
    app.run(debug=True)


@click.command()
@click.option("--debug", is_flag=True, help="Launch in Debug mode.")
def main(debug) -> None:
    if (debug):
        run_debug()
    else:
        run_normal()


if __name__ == "__main__":
    main()



from flask import render_template
from . import bp


@bp.app_errorhandler(400)
def bad_request(_):
    return render_template('400.html'), 401


@bp.app_errorhandler(401)
def access_denied_error(_):
    return render_template('401.html'), 401


@bp.app_errorhandler(404)
def not_found_error(_):
    return render_template('404.html'), 404


@bp.app_errorhandler(418)
def teapot(_):
    return render_template('418.html'), 418


@bp.app_errorhandler(500)
def internal_error(_):
    return render_template('500.html'), 500



"""Main backend file."""
import os
from flask import render_template, flash, redirect, url_for, request, jsonify, abort, send_file
from ...editor import get_scenes, create_project_dir, SectionId, populate_project, get_projects, get_project

from . import bp


@bp.route("/")
@bp.route("/index")
def index():
    return redirect(url_for("main.project_selection"))


@bp.route("/project_selection")
def project_selection():
    projects = get_projects()
    return render_template("project_selection.html", title="Project Selection", cwd=os.getcwd(), projects=projects)


@bp.route("/edit_project/<name>")
def edit_project(name: str):
    project_name, sections = get_project(os.path.join(name, "project.json"))
    if project_name is None:
        abort(404)
    return render_template("edit_project.html", title="Edit Project", name=name, sections=sections)


@bp.route("/serve_project_static/<name>/<path>")
def serve_project_static(name: str, path: str):
    """Since the projects are not in the Flask static folder, this function serves the videos and thumbnails instead."""
    abspath = os.path.abspath(os.path.join(name, path))
    # NOTE: this should never be used in an online environment, for a local one it's fine
    return send_file(abspath)


@ bp.route("/create_project")
def create_project():
    return redirect(url_for("main.set_project_name"))


@ bp.route("/create_project1")
def set_project_name():
    return render_template("set_project_name.html", title="Create New Project")


@ bp.route("/create_project2", methods=["POST"])
def section_selection():
    project_name = request.form["project_name"].strip()
    success, message = create_project_dir(project_name)
    if not success:
        flash(message, "danger")
        return redirect(url_for("main.set_project_name"))

    flash(message, "success")
    scenes = get_scenes()
    if len(scenes) == 0:
        flash("No sections were found. You must render you scene with the '--save_sections' flag. Refer to the documentation for more information.", "danger")
    return render_template("section_selection.html", title="Create New Project", scenes=scenes, project_name=project_name)


# ajax
@ bp.route("/create_project3", methods=["POST"])
def confirm_section_selection():
    project = request.json
    if project is None:
        abort(400)
    project_name = project["name"]
    sections = [SectionId(section["scene_id"], section["section_id"]) for section in project["sections"]]
    success = populate_project(project_name, sections)
    if success:
        flash(f"The project '{project_name}' has successfully been populated.", "success")
        return jsonify(success=True)
    else:
        # the flash will be taken care of by the client
        return jsonify(success=False)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['manim_editor',
 'manim_editor.app',
 'manim_editor.app.error',
 'manim_editor.app.main',
 'manim_editor.editor']

package_data = \
{'': ['*'],
 'manim_editor.app': ['static/img/*', 'templates/*'],
 'manim_editor.app.error': ['templates/*'],
 'manim_editor.app.main': ['templates/*']}

install_requires = \
['Flask>=2.0.2,<3.0.0',
 'click>=8.0.3,<9.0.0',
 'jsonschema>=4.1.2,<5.0.0',
 'waitress>=2.0.0,<3.0.0']

entry_points = \
{'console_scripts': ['manedit = manim_editor.__main__:main']}

setup_kwargs = {
    'name': 'manim-editor',
    'version': '0.1.0',
    'description': 'Editor and Presenter for Manim Generated Content',
    'long_description': None,
    'author': 'christopher-besch',
    'author_email': 'christopher.besch@gmx.de',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
